==> zed-sync-0.3.6.tgz: package/lib/utils.js <==
const colors = require('colors')
const { spawnSync, spawn, exec } = require('child_process')
const os = require('os')
const path = require('path')
const urllib = require('urllib-sync')
const fs = require('fs')
const cache = require('./cache')
const co = require('co')
const notifier = require('node-notifier');

const ZED_URL_CACHE_TTL = 5 * 24 * 3600 * 1000
const BRANCH_VERIFY_CACHE_TTL = 5 * 24 * 3600 * 1000

function throttle(func, interval) {
  interval = interval || 50
  const thisFn = throttle
  if (!thisFn.queue) {
    thisFn.makeTimer = function () {
      thisFn.timer = setInterval(function () {
        const func =  thisFn.queue.shift()
        if (func) {
          try {
            func()
          } catch (e) {
            console.error(e)
          }
        } else {
          clearInterval(thisFn.timer)
          thisFn.timer = null
        }
      }, interval)
    }
    thisFn.queue = []
    thisFn.makeTimer()
  }

  thisFn.queue.push(func)
  if (thisFn.timer === null) {
    thisFn.makeTimer()
  }
}

const logger = {
  argv: null,
  error() {
    if (!arguments.length) return
    const args = [].slice.call(arguments)
    notifier.notify({
      'title': 'zed-sync error:' + (logger.argv.p || logger.argv.url),
      'message': args.map(a => a.trim()).join(' '),
    });
    args[0] = colors.red(args[0])
    console.error.apply(null, args)
  },
  errorAndExit() {
    logger.error.apply(null, arguments)
    process.exit(1)
  },
  log() {
    let args = [].slice.call(arguments)
    if (process.env.DEBUG) {
      args = args.concat([Date.now()])
    }
    console.log.apply(null, args)
  },
  warn() {
    let args = [].slice.call(arguments)
    args[0] = colors.yellow(args[0])
    logger.log.apply(null, args)
  },
  success() {
    let args = [].slice.call(arguments)
    args[0] = colors.green(args[0])
    logger.log.apply(null, args)
  }
}

function notifyUpdate() {
  const updateNotifier = require('update-notifier')
  const pkg = require('../package.json')
  updateNotifier({
    pkg,
    updateCheckInterval: 1000 * 60 * 60 * 24, // 24 hours
  }).notify({ defer: false })
}

function SSHExec(user, cmd) {
  console.log('Exec command in doubandev2'.yellow, cmd, '...')
  const stdoutBufs = []
  const stderrBufs = []

  const p = logger.argv.localDae
    ? exec(cmd)
    : spawn('ssh', [`${user}@doubandev2`, '-t', `bash -i -c "${cmd.replace('"', '\\"')}"`])
  p.stdout.on('data', data => {
    data = Buffer.from(data)
    stdoutBufs.push(data)
    process.stdout.write(data)
  })
  p.stderr.on('data', data => {
    data = Buffer.from(data)
    stderrBufs.push(data)
    process.stdout.write(data)
  })
  const stdoutP = new Promise(resolve => p.stdout.on('end', () => resolve(Buffer.concat(stdoutBufs))))
  const stderrP = new Promise(resolve => p.stderr.on('end', () => resolve(Buffer.concat(stderrBufs))))
  return Promise.all([stdoutP, stderrP])
    .then(([stdout, stderr]) => ({stdout, stderr}))
}

function* getZedURLByPreInfo(preInfo, options={}) {
  const cacheKey = `url:${preInfo}`
  let url = cache.getKey(cacheKey)
  if (url && options.force !== true) {
    return url
  } else {
    console.log('Auto get zed url from pre:'.yellow, preInfo,'...')
  }

  const [user, pre] = preInfo.split('@')
  const ret = yield SSHExec(user, `dae pre edit --pre ${pre}`)
  const out = (ret.stdout.length ? ret.stdout : ret.stderr).toString('utf-8')
  url = (out.match(/(https?:\/\/[^\s]+)/) || [])[1]
  if (!url) {
    logger.error('Auto get zed url failed', ret.stdout + '', ret.stderr + '')
    process.exit(1)
  }
  cache.setKey(cacheKey, url, ZED_URL_CACHE_TTL)
  logger.log('Auto get zed url success:'.green, url)
  return url
}

function validateZedURL(zedURL, curBranch) {
  const skipInfo = 'If it\'s ok in your case, you can skip verify by \`-v=false\`.'
  const branchKey = `branch:${zedURL}`
  const lastBranch = cache.getKey(branchKey)
  if (!lastBranch) {
    cache.setKey(branchKey, curBranch, BRANCH_VERIFY_CACHE_TTL)
  } else if (lastBranch !== curBranch) {
    logger.errorAndExit(`Current branch didn\'t match last branch for this zed url, ${skipInfo}`)
  }
  let res
  try {
    res = urllib.request(zedURL)
  } catch (e) {
    logger.errorAndExit('Server is invalid', e.message, e.stack)
  }
  if (res.status !== 200) {
    logger.errorAndExit('Server is invalid', res.status, res.data.toString('utf-8'))
  }
  const readmeName = res.data.toString('utf-8').split('\n').filter((item) => item.match(/^README/i))[0]
  if (!readmeName) {
    logger.log('Cannot detect README file'.yellow)
    return
  }
  const remoteReadMe = urllib.request(zedURL + '/' + readmeName).data.toString('utf-8')
  let localReadMe
  try {
    localReadMe = fs.readFileSync(path.resolve(process.cwd(), readmeName), 'utf-8')
  } catch (e) {
    logger.error(e)
  }
  if (remoteReadMe !== localReadMe) {
    logger.error(`The remote and local ${readmeName}'s content doesn't match, this usually means you run this command in a wrong repository, or you just change your ${readmeName} file. ;)`)
  }
}

function sleep(ms) {
  return new Promise(resolve => setTimeout(resolve, ms))
}

module.exports = {
  logger, throttle, notifyUpdate, getZedURLByPreInfo,
  validateZedURL, cache, SSHExec, sleep
}

==> zed-sync-0.3.7.tgz: package/lib/utils.js <==
const colors = require('colors')
const { spawnSync, spawn, exec } = require('child_process')
const os = require('os')
const path = require('path')
const urllib = require('urllib-sync')
const fs = require('fs')
const cache = require('./cache')
const co = require('co')
const notifier = require('node-notifier');

const ZED_URL_CACHE_TTL = 5 * 24 * 3600 * 1000
const BRANCH_VERIFY_CACHE_TTL = 5 * 24 * 3600 * 1000

function throttle(func, interval) {
  interval = interval || 50
  const thisFn = throttle
  if (!thisFn.queue) {
    thisFn.makeTimer = function () {
      thisFn.timer = setInterval(function () {
        const func =  thisFn.queue.shift()
        if (func) {
          try {
            func()
          } catch (e) {
            console.error(e)
          }
        } else {
          clearInterval(thisFn.timer)
          thisFn.timer = null
        }
      }, interval)
    }
    thisFn.queue = []
    thisFn.makeTimer()
  }

  thisFn.queue.push(func)
  if (thisFn.timer === null) {
    thisFn.makeTimer()
  }
}

const logger = {
  argv: null,
  error() {
    if (!arguments.length) return
    const args = [].slice.call(arguments).map(String)
    notifier.notify({
      'title': 'zed-sync error:' + (logger.argv.p || logger.argv.url),
      'message': args.map(a => a.trim()).join(' '),
    });
    args[0] = colors.red(args[0])
    console.error.apply(null, args)
  },
  errorAndExit() {
    logger.error.apply(null, arguments)
    process.exit(1)
  },
  log() {
    let args = [].slice.call(arguments)
    if (process.env.DEBUG) {
      args = args.concat([Date.now()])
    }
    console.log.apply(null, args)
  },
  warn() {
    let args = [].slice.call(arguments)
    args[0] = colors.yellow(args[0])
    logger.log.apply(null, args)
  },
  success() {
    let args = [].slice.call(arguments)
    args[0] = colors.green(args[0])
    logger.log.apply(null, args)
  }
}

function notifyUpdate() {
  const updateNotifier = require('update-notifier')
  const pkg = require('../package.json')
  updateNotifier({
    pkg,
    updateCheckInterval: 1000 * 60 * 60 * 24, // 24 hours
  }).notify({ defer: false })
}

function SSHExec(user, cmd) {
  console.log('Exec command in doubandev2'.yellow, cmd, '...')
  const stdoutBufs = []
  const stderrBufs = []

  const p = logger.argv.localDae
    ? exec(cmd)
    : spawn('ssh', [`${user}@doubandev2`, '-t', `bash -i -c "${cmd.replace('"', '\\"')}"`])
  p.stdout.on('data', data => {
    data = Buffer.from(data)
    stdoutBufs.push(data)
    process.stdout.write(data)
  })
  p.stderr.on('data', data => {
    data = Buffer.from(data)
    stderrBufs.push(data)
    process.stdout.write(data)
  })
  const stdoutP = new Promise(resolve => p.stdout.on('end', () => resolve(Buffer.concat(stdoutBufs))))
  const stderrP = new Promise(resolve => p.stderr.on('end', () => resolve(Buffer.concat(stderrBufs))))
  return Promise.all([stdoutP, stderrP])
    .then(([stdout, stderr]) => ({stdout, stderr}))
}

function* getZedURLByPreInfo(preInfo, options={}) {
  const cacheKey = `url:${preInfo}`
  let url = cache.getKey(cacheKey)
  if (url && options.force !== true) {
    return url
  } else {
    console.log('Auto get zed url from pre:'.yellow, preInfo,'...')
  }

  const [user, pre] = preInfo.split('@')
  const ret = yield SSHExec(user, `dae pre edit --pre ${pre}`)
  const out = (ret.stdout.length ? ret.stdout : ret.stderr).toString('utf-8')
  url = (out.match(/(https?:\/\/[^\s]+)/) || [])[1]
  if (!url) {
    logger.error('Auto get zed url failed', ret.stdout + '', ret.stderr + '')
    process.exit(1)
  }
  cache.setKey(cacheKey, url, ZED_URL_CACHE_TTL)
  logger.log('Auto get zed url success:'.green, url)
  return url
}

function validateZedURL(zedURL, curBranch) {
  const skipInfo = 'If it\'s ok in your case, you can skip verify by \`-v=false\`.'
  const branchKey = `branch:${zedURL}`
  const lastBranch = cache.getKey(branchKey)
  if (!lastBranch) {
    cache.setKey(branchKey, curBranch, BRANCH_VERIFY_CACHE_TTL)
  } else if (lastBranch !== curBranch) {
    logger.errorAndExit(`Current branch didn\'t match last branch for this zed url, ${skipInfo}`)
  }
  let res
  try {
    res = urllib.request(zedURL)
  } catch (e) {
    logger.errorAndExit('Server is invalid', e.message, e.stack)
  }
  if (res.status !== 200) {
    logger.errorAndExit('Server is invalid', res.status, res.data.toString('utf-8'))
  }
  const readmeName = res.data.toString('utf-8').split('\n').filter((item) => item.match(/^README/i))[0]
  if (!readmeName) {
    logger.log('Cannot detect README file'.yellow)
    return
  }
  const remoteReadMe = urllib.request(zedURL + '/' + readmeName).data.toString('utf-8')
  let localReadMe
  try {
    localReadMe = fs.readFileSync(path.resolve(process.cwd(), readmeName), 'utf-8')
  } catch (e) {
    logger.error(e)
  }
  if (remoteReadMe !== localReadMe) {
    logger.error(`The remote and local ${readmeName}'s content doesn't match, this usually means you run this command in a wrong repository, or you just change your ${readmeName} file. ;)`)
  }
}

function sleep(ms) {
  return new Promise(resolve => setTimeout(resolve, ms))
}

module.exports = {
  logger, throttle, notifyUpdate, getZedURLByPreInfo,
  validateZedURL, cache, SSHExec, sleep
}
